using StudiApp.Models;
using StudiApp.Services.Facade;

namespace StudiApp.Test.Unit.ServicesTests.FacadeTests;

public class TimetableItemFacadeTests
{
    private TimetableItemFacade _timetableItemFacade = null!;

    [SetUp]
    public void Setup()
    {
        _timetableItemFacade = new TimetableItemFacade();
    }

    [Test]
    public void TestFromLectures()
    {
        // Arrange
        var lecture = new Lecture()
        {
            Id = 2,
            Name = "TestLecture1",
            DayTime = new DayTime()
            {
                DayOfWeek = DayOfWeek.Monday,
                StartTime = new TimeOnly(08, 10),
                EndTime = new TimeOnly(15, 03),
            },
        };

        var lectures = new List<Lecture>() { lecture };
        
        // Act
        var timetableItems = _timetableItemFacade.FromLectures(lectures);
        
        // Assert
        Assert.That(timetableItems, Has.Count.EqualTo(1));
        Assert.Multiple(() =>
        {
            Assert.That(timetableItems[0].Id, Is.EqualTo(lecture.Id));
            Assert.That(timetableItems[0].Name, Is.EqualTo(lecture.Name));
        });
    }

    [Test]
    public void TestFromTodos()
    {
        // Arrange
        var todo = new ToDo()
        {
            Id = 25,
            Name = "TestTodo",
            Description = "TestTodo",
            Due = new DateTime(2023, 05, 06),
        };

        var todos = new List<ToDo>() { todo };
        
        // Act
        var timetableItems = _timetableItemFacade.FromTodos(todos);
        
        // Assert
        Assert.That(timetableItems, Has.Count.EqualTo(1));
        Assert.Multiple(() =>
        {
            Assert.That(timetableItems[0].Id, Is.EqualTo(todo.Id));
            Assert.That(timetableItems[0].Name, Is.EqualTo(todo.Name));
        });
    }
}