﻿using Moq;
using StudiApp.ViewModels;

namespace StudiApp.Test.Unit.ViewModelsTests.CourseModule;

//This TestFixture is responsible for testing the base class of CourseEditViewModel and will not be retested for CourseNewViewModel
[TestFixture]
public class CourseEditViewModelTest
{
	[SetUp]
	public void SetUp()
	{
		_helper = new CourseModuleTestHelper();
		_vm = new CoursesEditViewModel(_helper.ViewModelService.Object)
		{
			Id = 1
		};
	}

	private CoursesEditViewModel _vm = null!;
	private CourseModuleTestHelper _helper = null!;

	[Test]
	public void AppearingCommandTest()
	{
		_vm.AppearingCommand.Execute(null);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.GetCourse(_vm.Id), Times.Once);
		_helper.RepositoryManager.Verify(r => r.GetAllSemesters(null), Times.Once);

		Assert.Multiple(() =>
		{
			//Assert Course
			Assert.That(_vm.Course.Name, Is.EqualTo("ActiveCourse"));
			Assert.That(_vm.Course.Id, Is.EqualTo(1));
			Assert.That(_vm.Course.LecturerName, Is.EqualTo("LecturerName"));
			Assert.That(_vm.Course.Semester, Is.EqualTo(_helper.SemesterActive));
			Assert.That(_vm.Course.Description, Is.EqualTo("Description"));
			Assert.That(_vm.Course.Ects, Is.EqualTo(4));
			Assert.That(_vm.Course.Marks, Has.Count.EqualTo(2));
			Assert.That(_vm.Course.Lectures, Has.Count.EqualTo(2));
			Assert.That(_vm.Course.ToDos, Is.EqualTo(null));

			//Assert SelectedSemester
			Assert.That(_vm.SelectedSemester, Is.EqualTo(_helper.SemesterActive));

			//Assert Lectures
			Assert.That(_vm.Lectures, Has.Count.EqualTo(2));
			Assert.That(_vm.Lectures.First().DayTime.DayOfWeek, Is.EqualTo(DayOfWeek.Friday));
			Assert.That(_vm.Lectures.First().DayTime.StartTime, Is.EqualTo(new TimeOnly(12, 0)));
			Assert.That(_vm.Lectures.First().DayTime.EndTime, Is.EqualTo(new TimeOnly(13, 0)));
			Assert.That(_vm.Lectures.First().Name, Is.EqualTo("Lecture1"));
			Assert.That(_vm.Lectures.First().Room, Is.EqualTo("1"));
			Assert.That(_vm.Lectures.Last().DayTime.DayOfWeek, Is.EqualTo(DayOfWeek.Monday));
			Assert.That(_vm.Lectures.Last().DayTime.StartTime, Is.EqualTo(new TimeOnly(12, 0)));
			Assert.That(_vm.Lectures.Last().DayTime.EndTime, Is.EqualTo(new TimeOnly(13, 0)));
			Assert.That(_vm.Lectures.Last().Name, Is.EqualTo("Lecture2"));
			Assert.That(_vm.Lectures.Last().Room, Is.EqualTo("2"));

			//Assert WeekDays
			Assert.That(_vm.WeekDays[0], Is.EqualTo(DayOfWeek.Sunday));
			Assert.That(_vm.WeekDays[1], Is.EqualTo(DayOfWeek.Monday));
			Assert.That(_vm.WeekDays[2], Is.EqualTo(DayOfWeek.Tuesday));
			Assert.That(_vm.WeekDays[3], Is.EqualTo(DayOfWeek.Wednesday));
			Assert.That(_vm.WeekDays[4], Is.EqualTo(DayOfWeek.Thursday));
			Assert.That(_vm.WeekDays[5], Is.EqualTo(DayOfWeek.Friday));
			Assert.That(_vm.WeekDays[6], Is.EqualTo(DayOfWeek.Saturday));

			//Assert Semesters
			Assert.That(_vm.Semesters.First().Name, Is.EqualTo("Inactive"));
			Assert.That(_vm.Semesters.First().Id, Is.EqualTo(1));
			Assert.That(_vm.Semesters.Last().Name, Is.EqualTo("Active"));
			Assert.That(_vm.Semesters.Last().Id, Is.EqualTo(2));
		});
	}

	[Test]
	public void SaveCommandIsValidTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.SaveCommand.Execute(null);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.SaveCourse(_vm.Course), Times.Once);
		_helper.NavigationManager.Verify(n => n.NavigateBack(), Times.Once);
	}

	[Test]
	public void SaveCommandNotValidTest()
	{
		_vm.AppearingCommand.Execute(null);

		//Make Course Invalid
		_vm.Course.Name = null;

		_vm.SaveCommand.Execute(null);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.SaveCourse(_vm.Course), Times.Never);
		_helper.NavigationManager.Verify(n => n.NavigateBack(), Times.Never);
	}

	[Test]
	public void AddLectureCommandTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.AddLectureCommand.Execute(null);

		//Assertions
		Assert.Multiple(() =>
		{
			Assert.That(_vm.Lectures, Has.Count.EqualTo(3));
			Assert.That(_vm.Lectures.Last().Name, Is.EqualTo(null));
			Assert.That(_vm.Lectures.Last().DayTime.DayOfWeek, Is.EqualTo(DayOfWeek.Monday));
		});
	}

	[Test]
	public void DeleteLectureCommandTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.DeleteLectureCommand.Execute(_helper.Lecture2);

		//Assertions
		_helper.DialogService.Verify(d => d.ShowConfirmationAsync("Are you sure you want to remove this lecture?"),
			Times.Once);

		Assert.Multiple(() =>
		{
			Assert.That(_vm.Lectures, Has.Count.EqualTo(1));
			Assert.That(_vm.Lectures.First().Name, Is.EqualTo("Lecture1"));
			Assert.That(_vm.Lectures.First().Room, Is.EqualTo("1"));
			Assert.That(_vm.Lectures.First().DayTime.DayOfWeek, Is.EqualTo(DayOfWeek.Friday));
			Assert.That(_vm.Lectures.First().DayTime.StartTime, Is.EqualTo(new TimeOnly(12, 0)));
			Assert.That(_vm.Lectures.First().DayTime.EndTime, Is.EqualTo(new TimeOnly(13, 0)));
		});
	}
}