﻿using Microsoft.Extensions.Logging;
using Moq;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.InputValidator;
using StudiApp.Services.ViewModelService;

namespace StudiApp.Test.Unit.ViewModelsTests.CourseModule;

internal class CourseModuleTestHelper
{
	internal Course CourseActive;
	internal Course CourseNotActive;
	internal Mock<IDialogService> DialogService;
	internal IInputValidator InputValidator;
	internal Lecture Lecture1;
	internal Lecture Lecture2;
	internal Mock<ILogger<ViewModelService>> Logger;
	internal Mark Mark1;
	internal Mark Mark2;
	internal Mock<INavigationManager> NavigationManager;
	internal Mock<IRepositoryManager> RepositoryManager;
	internal Semester SemesterActive;
	internal Semester SemesterNotActive;
	internal Mock<IViewModelService> ViewModelService;

	internal CourseModuleTestHelper()
	{
		//Setup Semesters
		SemesterNotActive = new Semester
		{
			Name = "Inactive",
			DateRange = new DateRange
			{
				Start = new DateTime(1999, 01, 01),
				End = new DateTime(2000, 01, 01)
			},
			Id = 1
		};

		//Add and subtracts 100 days from Datetime.Now (Is always active)
		SemesterActive = new Semester
		{
			Name = "Active",
			DateRange = new DateRange
			{
				Start = DateTime.Now.Subtract(new TimeSpan(100, 0, 0, 0)),
				End = DateTime.Now.Add(new TimeSpan(100, 0, 0, 0))
			},
			Id = 2
		};

		var allSemesters = new List<Semester> {SemesterNotActive, SemesterActive};

		//Setup Lectures
		Lecture1 = new Lecture
		{
			Name = "Lecture1",
			Room = "1",
			DayTime = new DayTime
			{
				DayOfWeek = DayOfWeek.Friday,
				StartTime = new TimeOnly(12, 0),
				EndTime = new TimeOnly(13, 0)
			}
		};

		Lecture2 = new Lecture
		{
			Name = "Lecture2",
			Room = "2",
			DayTime = new DayTime
			{
				DayOfWeek = DayOfWeek.Monday,
				StartTime = new TimeOnly(12, 0),
				EndTime = new TimeOnly(13, 0)
			}
		};

		//Setup Marks
		Mark1 = new Mark
		{
			Value = 3.0,
			Percentage = 0.5
		};

		Mark2 = new Mark
		{
			Value = 5.0,
			Percentage = 0.5
		};

		//Setup Courses
		CourseActive = new Course
		{
			Name = "ActiveCourse",
			Semester = SemesterActive,
			SemesterId = 2,
			LecturerName = "LecturerName",
			Description = "Description",
			Ects = 4,
			Id = 1,
			Lectures = new List<Lecture> {Lecture1, Lecture2},
			Marks = new List<Mark> {Mark1, Mark2}
		};

		CourseNotActive = new Course
		{
			Name = "NotActiveCourse",
			Semester = SemesterNotActive,
			LecturerName = "LecturerName",
			Description = "Description",
			Ects = 4,
			Id = 2,
			Lectures = new List<Lecture> {Lecture1, Lecture2},
			Marks = new List<Mark> {Mark1, Mark2}
		};

		var allCourses = new List<Course> {CourseActive, CourseNotActive};

		//Setup Mocks
		DialogService = new Mock<IDialogService>();
		InputValidator = new InputValidator();
		NavigationManager = new Mock<INavigationManager>();
		RepositoryManager = new Mock<IRepositoryManager>();
		Logger = new Mock<ILogger<ViewModelService>>();

		ViewModelService = new Mock<IViewModelService>();

		//Setup RepositoryManager
		RepositoryManager.Setup(r => r.GetAllSemesters(null)).Returns(Task.FromResult(allSemesters));
		RepositoryManager.Setup(r => r.GetAllCourses(null)).Returns(Task.FromResult(allCourses));
		RepositoryManager.Setup(r => r.GetCourse(CourseActive.Id)).Returns(Task.FromResult(CourseActive));
		RepositoryManager.Setup(r => r.DeleteCourse(CourseActive)).Returns(Task.FromResult(true));

		//Setup DialogService
		DialogService.Setup(d => d.ShowConfirmationAsync(It.IsAny<string>())).Returns(Task.FromResult(true));

		//Setup ViewModelService
		ViewModelService.Setup(v => v.DialogService).Returns(DialogService.Object);
		ViewModelService.Setup(v => v.InputValidator).Returns(InputValidator);
		ViewModelService.Setup(v => v.NavigationManager).Returns(NavigationManager.Object);
		ViewModelService.Setup(v => v.Logger).Returns(Logger.Object);
		ViewModelService.Setup(v => v.RepositoryManager).Returns(RepositoryManager.Object);
	}
}