﻿using StudiApp.Models;

namespace StudiApp.Services.InputValidator.Strategy;

public class SemesterValidatorStrategy : ValidatorStrategyBase
{
	private const string SemesterModelName = nameof(Semester);

	private readonly Semester _semester;

	public SemesterValidatorStrategy(Semester semester)
	{
		_semester = semester;
	}

	public override ValidatorItem IsValid()
	{
		var semesterNameValidatorItem = IsSemesterNameValid();
		var semesterDateRangeValidatorItem = IsDateRangeValid();

		var validatorItemList = new List<ValidatorItem>
		{
			semesterNameValidatorItem,
			semesterDateRangeValidatorItem
		};

		return MergeValidatorItems(validatorItemList);
	}

	private ValidatorItem IsDateRangeValid()
	{
		return _semester.DateRange.Start < _semester.DateRange.End
			? new ValidatorItem(true)
			: new ValidatorItem(false, GetErrorMessageTime(SemesterModelName));
	}

	private ValidatorItem IsSemesterNameValid()
	{
		return _semester.Name is {Length: <= 100 and > 1}
			? new ValidatorItem(true)
			: new ValidatorItem(false, GetErrorMessageName(SemesterModelName));
	}
}