﻿using StudiApp.Models;

namespace StudiApp.Services.InputValidator.Strategy;

public class TodoValidatorStrategy : ValidatorStrategyBase
{
	private const string ToDoModelName = nameof(ToDo);
	private readonly ToDo _todo;

	public TodoValidatorStrategy(ToDo todo)
	{
		_todo = todo;
	}

	public override ValidatorItem IsValid()
	{
		var todoNameValidatorItem = IsTodoNameValid();
		var todoDescriptionValidatorItem = IsTodoDescriptionValid();

		var validatorItemList = new List<ValidatorItem>
		{
			todoNameValidatorItem,
			todoDescriptionValidatorItem
		};

		return MergeValidatorItems(validatorItemList);
	}

	private ValidatorItem IsTodoDescriptionValid()
	{
		return _todo.Description is null or {Length: <= DescriptionStringMaxValue}
			? new ValidatorItem(true)
			: new ValidatorItem(false, GetErrorMessageDescription(ToDoModelName));
	}


	private ValidatorItem IsTodoNameValid()
	{
		return _todo.Name is {Length: > NameStringMinValue and <= NameStringMaxValue}
			? new ValidatorItem(true)
			: new ValidatorItem(false, GetErrorMessageName(ToDoModelName));
	}
}