﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.Converters;
using StudiApp.Views;

namespace StudiApp.ViewModels;

[QueryProperty("Id", "Id")]
public partial class CoursesDetailViewModel : ObservableObject
{
	private readonly IViewModelService _viewModelService;

	[ObservableProperty] private double _averageMark;
	[ObservableProperty] private Course _course;
	[ObservableProperty] private ObservableCollection<Lecture> _nextLectures = new();

	public int Id { get; set; }

	public CoursesDetailViewModel(IViewModelService viewModelService)
	{
		_viewModelService = viewModelService;
	}

	private ObservableCollection<Lecture> GetLectures()
	{
		var nextLectures = new ObservableCollection<Lecture>();

		foreach (var lecture in Course.Lectures) nextLectures.Add(lecture);

		_viewModelService.Logger.LogInformation("Next Lectures loaded");
		return nextLectures;
	}

	private double GetAverageMark()
	{
		return AverageMarkConverter.MarksToAverage(Course.Marks);
	}

	[RelayCommand]
	public async Task Delete()
	{
		var result = await _viewModelService.DialogService.ShowConfirmationAsync(
				$"Are you sure you want to delete this course ({Course.Name})?");

		if (result)
		{
			await _viewModelService.RepositoryManager.DeleteCourse(Course);
			_viewModelService.Logger.LogInformation("Course {CourseName} deleted", Course.Name);
			await _viewModelService.NavigationManager.NavigateBack();
		}
	}

	[RelayCommand]
	public async Task OnAppearing()
	{
		Course = await _viewModelService.RepositoryManager.GetCourse(Id);
		_viewModelService.Logger.LogInformation("Course {CourseName} loaded", Course.Name);

		NextLectures = GetLectures();
		AverageMark = GetAverageMark();

		_viewModelService.Logger.LogInformation("Detail Page for {CourseName} loaded", Course.Name);
	}

	[RelayCommand]
	public async Task Edit(Course c)
	{
		await _viewModelService.NavigationManager.NavigateTo(nameof(CoursesEditView), "Id", c.Id);
		_viewModelService.Logger.LogInformation("Navigating to {ViewName}", nameof(CoursesEditView));
	}
}