﻿using CommunityToolkit.Mvvm.ComponentModel;
using StudiApp.Models;
using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Services.InputValidator.Strategy;
using StudiApp.Services.Converters;
using StudiApp.Services.ViewModelService;

namespace StudiApp.ViewModels
{
    public abstract partial class MarksOperationsBaseViewModel : ObservableObject
    {
        protected readonly IViewModelService ViewModelService;

        [ObservableProperty] private Mark _mark;
        [ObservableProperty] private string _percentage;
        [ObservableProperty] private ObservableCollection<Course> _courses;

        protected MarksOperationsBaseViewModel(IViewModelService viewModelService)
        {
            ViewModelService = viewModelService;
        }

        private async Task<ObservableCollection<Course>> InitializeCourses()
        {
            var courses = new ObservableCollection<Course>();

            foreach (var course in await ViewModelService.RepositoryManager.GetAllCourses()) courses.Add(course);

            ViewModelService.Logger.LogInformation("Courses initialized");
            return courses;
        }

        protected async Task OnAppearing()
        {
            Courses = await InitializeCourses();
            ViewModelService.Logger.LogInformation($"{nameof(MarksOperationsBaseViewModel)} initialized");
        }

        protected void SetMark(Mark mark)
        {
            Mark = mark;
        }

        protected void SetPercentage(string percentage)
        {
            Percentage = percentage;
        }

        [RelayCommand]
        public async Task Save()
        {
            try
            {
                Mark.Percentage = PercentageConverter.ConvertToDouble(Percentage);
                var validatorResult = ViewModelService.InputValidator.IsValid(new MarkValidatorStrategy(Mark));

                if (validatorResult.IsValid)
                {
                    await ViewModelService.RepositoryManager.SaveMark(Mark);
                    ViewModelService.Logger.LogInformation("Mark saved");
                    await ViewModelService.NavigationManager.NavigateBack();
                }
                else
                {
                    await ViewModelService.DialogService.ShowAlertAsync(validatorResult.ErrorMessage);
                    ViewModelService.Logger.LogInformation("Mark couldn't save because of errors: {ErrorMessage}", validatorResult.ErrorMessage);
                }
            }
            catch (ArgumentException e)
            {
                await ViewModelService.DialogService.ShowAlertAsync(e.Message);
            }
            
        }
    }
}
