﻿using CommunityToolkit.Mvvm.ComponentModel;
using StudiApp.Models;
using Syncfusion.Maui.Scheduler;
using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.Input;
using StudiApp.Interfaces;

namespace StudiApp.ViewModels;

public partial class TimetableViewModel : ObservableObject
{

    [ObservableProperty]
    private ObservableCollection<TimetableItem> _timetableItems;

    private readonly IRepositoryManager _repositoryManager;
    private readonly ITimetableItemFacade _timetableItemFacade;

    public TimetableViewModel(IRepositoryManager repositoryManager, ITimetableItemFacade timetableItemFacade)
    {
        _repositoryManager = repositoryManager;
        _timetableItemFacade = timetableItemFacade;

        TimetableItems = new ObservableCollection<TimetableItem>();
    }

    private async Task<Semester> GetCurrentSemester()
    {
        var semesters = await _repositoryManager.GetAllSemesters();
        return Semester.GetCurrent(semesters);
    }

    private static List<Lecture> GetLectures(Semester semester)
    {
        var lectures = new List<Lecture>();
        foreach (var course in semester.Courses)
        {
            lectures.AddRange(course.Lectures);
        }

        return lectures;
    }

    [RelayCommand]
    public void OnAppearing()
    {
        GetCurrentSemester().ContinueWith(async semesterTask =>
        {
            var timetableItems = new ObservableCollection<TimetableItem>();
            var semester = semesterTask.Result;

            var lectureTimetableItems = _timetableItemFacade.FromLectures(GetLectures(semester));
            foreach (var timetableItem in lectureTimetableItems)
            {
                timetableItems.Add(timetableItem);
            }
            
            var todos = await _repositoryManager.GetAllToDos(t => !t.IsCompleted);
            var todoTimetableItems = _timetableItemFacade.FromTodos(todos);
            foreach (var timetableItem in todoTimetableItems)
            {
                timetableItems.Add(timetableItem);
            }

            TimetableItems = timetableItems;
        });
    }
}
